const axios = require('axios')
const config = require('@config/index')
const qs = require('querystring')

const queryYoutubeName = async (id = '') => {
  if (typeof id !== 'string' || id.trim().length === 0) return null
  id = id.trim()
  let url = `https://www.googleapis.com/youtube/v3/channels`
  let params = {
    id,
    part: `snippet`,
    key: config.google.apikey
  }
  try {
    let result = await axios({
      url,
      method: 'get',
      params
    })

    if (!result || !('data' in result) || !('items' in result.data) || !Array.isArray(result.data.items) || result.data.items.length === 0) return null
    let data = result.data.items[0]
    console.log('youtube data ::: ', JSON.stringify(data, null, 2))
    if (!('snippet' in data) || !('title' in data.snippet)) return null
    return data.snippet.title
  } catch (err) {
    console.log(err)
    return null
  }
}

const subYoutube = async (id = '') => {
  let url = 'https://pubsubhubbub.appspot.com/subscribe'
  if (typeof id !== 'string' || id.trim().length === 0) return null
  id = id.trim()

  let data = {
    'hub.mode': 'subscribe',
    'hub.verify': 'async',
    'hub.topic': `https://www.youtube.com/xml/feeds/videos.xml?channel_id=${id}`,
    'hub.callback': `${config.url.replace(/\/$/, '')}/google/youtube/webhook?id=${id}`,
    'hub.lease_seconds': 86400
  }

  try {
    await axios({
      headers: {
        'Content-Type': 'application/x-www-form-urlencoded'
      },
      data: qs.stringify(data),
      url,
      method: 'post'
    })
  } catch (err) {
    console.log(err)
  }
  return null
}

module.exports = {
  subYoutube,
  queryYoutubeName
}
